/*
 * UnitTestBPlusTree.cpp
 *
 *  Created on: Sep 12, 2010
 *      Author: data
 */

#include "UnitTestBPlusTree.h"
using namespace std;

UnitTestBPlusTree::UnitTestBPlusTree() {
	// TODO Auto-generated constructor stub

}

UnitTestBPlusTree::~UnitTestBPlusTree() {
	// TODO Auto-generated destructor stub
}

int UnitTestBPlusTree::main_b_plus_tree(int argc, char** argv) {

	iterator_test_6();
	return 0;
}

void UnitTestBPlusTree::iterator_test_1() {
	typedef stx::btree_multiset<unsigned int,
			std::less<unsigned int>, struct traits_nodebug> btree_type;

	std::vector<unsigned int> vector;

	srand(34234235);
	for(unsigned int i = 0; i < 3200; i++)
	{
		vector.push_back( rand() % 1000 );
	}

	assert( vector.size() == 3200 );

	// test construction and insert(iter, iter) function
	btree_type bt(vector.begin(), vector.end());

	assert( bt.size() == 3200 );

	// copy for later use
	btree_type bt2 = bt;

	// empty out the first bt
	srand(34234235);
	for(unsigned int i = 0; i < 3200; i++)
	{
		assert(bt.size() == 3200 - i);
		assert( bt.erase_one(rand() % 1000) );
		assert(bt.size() == 3200 - i - 1);
	}

	assert( bt.empty() );

	// copy btree values back to a vector

	std::vector<unsigned int> vector2;
	vector2.assign( bt2.begin(), bt2.end() );

	// afer sorting the vector, the two must be the same
	std::sort(vector.begin(), vector.end());

	assert( vector == vector2 );

	// test reverse iterator
	vector2.clear();
	vector2.assign( bt2.rbegin(), bt2.rend() );

	std::reverse(vector.begin(), vector.end());

	btree_type::reverse_iterator ri = bt2.rbegin();
	for(unsigned int i = 0; i < vector2.size(); ++i)
	{
		assert( vector[i] == vector2[i] );
		assert( vector[i] == *ri );

		ri++;
	}

	assert( ri == bt2.rend() );
}

void UnitTestBPlusTree::iterator_test_2() {
	typedef stx::btree_multimap<unsigned int, unsigned int,
			std::less<unsigned int>, struct traits_nodebug> btree_type;

	std::vector< btree_type::value_type > vector;

	srand(34234235);
	for(unsigned int i = 0; i < 3200; i++)
	{
		vector.push_back( btree_type::value_type(rand() % 1000, 0) );
	}

	assert( vector.size() == 3200 );

	// test construction and insert(iter, iter) function
	btree_type bt(vector.begin(), vector.end());

	assert( bt.size() == 3200 );

	// copy for later use
	btree_type bt2 = bt;

	// empty out the first bt
	srand(34234235);
	for(unsigned int i = 0; i < 3200; i++)
	{
		assert(bt.size() == 3200 - i);
		assert( bt.erase_one(rand() % 1000) );
		assert(bt.size() == 3200 - i - 1);
	}

	assert( bt.empty() );

	// copy btree values back to a vector

	std::vector< btree_type::value_type > vector2;
	vector2.assign( bt2.begin(), bt2.end() );

	// afer sorting the vector, the two must be the same
	std::sort(vector.begin(), vector.end());

	assert( vector == vector2 );

	// test reverse iterator
	vector2.clear();
	vector2.assign( bt2.rbegin(), bt2.rend() );

	std::reverse(vector.begin(), vector.end());

	btree_type::reverse_iterator ri = bt2.rbegin();
	for(unsigned int i = 0; i < vector2.size(); ++i, ++ri)
	{
		assert( vector[i].first == vector2[i].first );
		assert( vector[i].first == ri->first );
		assert( vector[i].second == ri->second );
	}

	assert( ri == bt2.rend() );
}

void UnitTestBPlusTree::iterator_test_3() {
	typedef stx::btree_map<unsigned int, unsigned int,
			std::less<unsigned int>, struct traits_nodebug> btree_type;

	btree_type map;

	unsigned int maxnum = 1000;

	for(unsigned int i = 0; i < maxnum; ++i)
	{
		map.insert( std::make_pair(i, i*3) );
	}

	{ // test iterator prefix++
		unsigned int nownum = 0;

		for(btree_type::iterator i = map.begin();
				i != map.end(); ++i)
		{
			assert( nownum == i->first );
			assert( nownum * 3 == i->second );

			nownum++;
		}

		assert(nownum == maxnum);
	}

	{ // test iterator prefix--
		unsigned int nownum = maxnum;

		btree_type::iterator i;
		for(i = --map.end(); i != map.begin(); --i)
		{
			nownum--;

			assert( nownum == i->first );
			assert( nownum * 3 == i->second );
		}

		nownum--;

		assert( nownum == i->first );
		assert( nownum * 3 == i->second );

		assert(nownum == 0);
	}

	{ // test const_iterator prefix++
		unsigned int nownum = 0;

		for(btree_type::const_iterator i = map.begin();
				i != map.end(); ++i)
		{
			assert( nownum == i->first );
			assert( nownum * 3 == i->second );

			nownum++;
		}

		assert(nownum == maxnum);
	}

	{ // test const_iterator prefix--
		unsigned int nownum = maxnum;

		btree_type::const_iterator i;
		for(i = --map.end(); i != map.begin(); --i)
		{
			nownum--;

			assert( nownum == i->first );
			assert( nownum * 3 == i->second );
		}

		nownum--;

		assert( nownum == i->first );
		assert( nownum * 3 == i->second );

		assert(nownum == 0);
	}

	{ // test reverse_iterator prefix++
		unsigned int nownum = maxnum;

		for(btree_type::reverse_iterator i = map.rbegin();
				i != map.rend(); ++i)
		{
			nownum--;

			assert( nownum == i->first );
			assert( nownum * 3 == i->second );
		}

		assert(nownum == 0);
	}

	{ // test reverse_iterator prefix--
		unsigned int nownum = 0;

		btree_type::reverse_iterator i;
		for(i = --map.rend(); i != map.rbegin(); --i)
		{
			assert( nownum == i->first );
			assert( nownum * 3 == i->second );

			nownum++;
		}

		assert( nownum == i->first );
		assert( nownum * 3 == i->second );

		nownum++;

		assert(nownum == maxnum);
	}

	{ // test const_reverse_iterator prefix++
		unsigned int nownum = maxnum;

		for(btree_type::const_reverse_iterator i = map.rbegin();
				i != map.rend(); ++i)
		{
			nownum--;

			assert( nownum == i->first );
			assert( nownum * 3 == i->second );
		}

		assert(nownum == 0);
	}

	{ // test const_reverse_iterator prefix--
		unsigned int nownum = 0;

		btree_type::const_reverse_iterator i;
		for(i = --map.rend(); i != map.rbegin(); --i)
		{
			assert( nownum == i->first );
			assert( nownum * 3 == i->second );

			nownum++;
		}

		assert( nownum == i->first );
		assert( nownum * 3 == i->second );

		nownum++;

		assert(nownum == maxnum);
	}

	// postfix

	{ // test iterator postfix++
		unsigned int nownum = 0;

		for(btree_type::iterator i = map.begin();
				i != map.end(); i++)
		{
			assert( nownum == i->first );
			assert( nownum * 3 == i->second );

			nownum++;
		}

		assert(nownum == maxnum);
	}

	{ // test iterator postfix--
		unsigned int nownum = maxnum;

		btree_type::iterator i;
		for(i = --map.end(); i != map.begin(); i--)
		{
			nownum--;

			assert( nownum == i->first );
			assert( nownum * 3 == i->second );
		}

		nownum--;

		assert( nownum == i->first );
		assert( nownum * 3 == i->second );

		assert(nownum == 0);
	}

	{ // test const_iterator postfix++
		unsigned int nownum = 0;

		for(btree_type::const_iterator i = map.begin();
				i != map.end(); i++)
		{
			assert( nownum == i->first );
			assert( nownum * 3 == i->second );

			nownum++;
		}

		assert(nownum == maxnum);
	}

	{ // test const_iterator postfix--
		unsigned int nownum = maxnum;

		btree_type::const_iterator i;
		for(i = --map.end(); i != map.begin(); i--)
		{
			nownum--;

			assert( nownum == i->first );
			assert( nownum * 3 == i->second );
		}

		nownum--;

		assert( nownum == i->first );
		assert( nownum * 3 == i->second );

		assert(nownum == 0);
	}

	{ // test reverse_iterator postfix++
		unsigned int nownum = maxnum;

		for(btree_type::reverse_iterator i = map.rbegin();
				i != map.rend(); i++)
		{
			nownum--;

			assert( nownum == i->first );
			assert( nownum * 3 == i->second );
		}

		assert(nownum == 0);
	}

	{ // test reverse_iterator postfix--
		unsigned int nownum = 0;

		btree_type::reverse_iterator i;
		for(i = --map.rend(); i != map.rbegin(); i--)
		{
			assert( nownum == i->first );
			assert( nownum * 3 == i->second );

			nownum++;
		}

		assert( nownum == i->first );
		assert( nownum * 3 == i->second );

		nownum++;

		assert(nownum == maxnum);
	}

	{ // test const_reverse_iterator postfix++
		unsigned int nownum = maxnum;

		for(btree_type::const_reverse_iterator i = map.rbegin();
				i != map.rend(); i++)
		{
			nownum--;

			assert( nownum == i->first );
			assert( nownum * 3 == i->second );
		}

		assert(nownum == 0);
	}

	{ // test const_reverse_iterator postfix--
		unsigned int nownum = 0;

		btree_type::const_reverse_iterator i;
		for(i = --map.rend(); i != map.rbegin(); i--)
		{
			assert( nownum == i->first );
			assert( nownum * 3 == i->second );

			nownum++;
		}

		assert( nownum == i->first );
		assert( nownum * 3 == i->second );

		nownum++;

		assert(nownum == maxnum);
	}
}

void UnitTestBPlusTree::iterator_test_4() {
	typedef stx::btree_set<unsigned int,
			std::less<unsigned int>, struct traits_nodebug> btree_type;

	btree_type set;

	unsigned int maxnum = 1000;

	for(unsigned int i = 0; i < maxnum; ++i)
	{
		set.insert(i);
	}

	{ // test iterator prefix++
		unsigned int nownum = 0;

		for(btree_type::iterator i = set.begin();
				i != set.end(); ++i)
		{
			assert( nownum == *i );
			nownum++;
		}

		assert(nownum == maxnum);
	}

	{ // test iterator prefix--
		unsigned int nownum = maxnum;

		btree_type::iterator i;
		for(i = --set.end(); i != set.begin(); --i)
		{
			assert( --nownum == *i );
		}

		assert( --nownum == *i );

		assert(nownum == 0);
	}

	{ // test const_iterator prefix++
		unsigned int nownum = 0;

		for(btree_type::const_iterator i = set.begin();
				i != set.end(); ++i)
		{
			assert( nownum++ == *i );
		}

		assert(nownum == maxnum);
	}

	{ // test const_iterator prefix--
		unsigned int nownum = maxnum;

		btree_type::const_iterator i;
		for(i = --set.end(); i != set.begin(); --i)
		{
			assert( --nownum == *i );
		}

		assert( --nownum == *i );

		assert(nownum == 0);
	}

	{ // test reverse_iterator prefix++
		unsigned int nownum = maxnum;

		for(btree_type::reverse_iterator i = set.rbegin();
				i != set.rend(); ++i)
		{
			assert( --nownum == *i );
		}

		assert(nownum == 0);
	}

	{ // test reverse_iterator prefix--
		unsigned int nownum = 0;

		btree_type::reverse_iterator i;
		for(i = --set.rend(); i != set.rbegin(); --i)
		{
			assert( nownum++ == *i );
		}

		assert( nownum++ == *i );

		assert(nownum == maxnum);
	}

	{ // test const_reverse_iterator prefix++
		unsigned int nownum = maxnum;

		for(btree_type::const_reverse_iterator i = set.rbegin();
				i != set.rend(); ++i)
		{
			assert( --nownum == *i );
		}

		assert(nownum == 0);
	}

	{ // test const_reverse_iterator prefix--
		unsigned int nownum = 0;

		btree_type::const_reverse_iterator i;
		for(i = --set.rend(); i != set.rbegin(); --i)
		{
			assert( nownum++ == *i );
		}

		assert( nownum++ == *i );

		assert(nownum == maxnum);
	}

	// postfix

	{ // test iterator postfix++
		unsigned int nownum = 0;

		for(btree_type::iterator i = set.begin();
				i != set.end(); i++)
		{
			assert( nownum++ == *i );
		}

		assert(nownum == maxnum);
	}

	{ // test iterator postfix--
		unsigned int nownum = maxnum;

		btree_type::iterator i;
		for(i = --set.end(); i != set.begin(); i--)
		{

			assert( --nownum == *i );
		}

		assert( --nownum == *i );

		assert(nownum == 0);
	}

	{ // test const_iterator postfix++
		unsigned int nownum = 0;

		for(btree_type::const_iterator i = set.begin();
				i != set.end(); i++)
		{
			assert( nownum++ == *i );
		}

		assert(nownum == maxnum);
	}

	{ // test const_iterator postfix--
		unsigned int nownum = maxnum;

		btree_type::const_iterator i;
		for(i = --set.end(); i != set.begin(); i--)
		{
			assert( --nownum == *i );
		}

		assert( --nownum == *i );

		assert(nownum == 0);
	}

	{ // test reverse_iterator postfix++
		unsigned int nownum = maxnum;

		for(btree_type::reverse_iterator i = set.rbegin();
				i != set.rend(); i++)
		{
			assert( --nownum == *i );
		}

		assert(nownum == 0);
	}

	{ // test reverse_iterator postfix--
		unsigned int nownum = 0;

		btree_type::reverse_iterator i;
		for(i = --set.rend(); i != set.rbegin(); i--)
		{
			assert( nownum++ == *i );
		}

		assert( nownum++ == *i );

		assert(nownum == maxnum);
	}

	{ // test const_reverse_iterator postfix++
		unsigned int nownum = maxnum;

		for(btree_type::const_reverse_iterator i = set.rbegin();
				i != set.rend(); i++)
		{
			assert( --nownum == *i );
		}

		assert(nownum == 0);
	}

	{ // test const_reverse_iterator postfix--
		unsigned int nownum = 0;

		btree_type::const_reverse_iterator i;
		for(i = --set.rend(); i != set.rbegin(); i--)
		{
			assert( nownum++ == *i );
		}

		assert( nownum++ == *i );

		assert(nownum == maxnum);
	}
}

void UnitTestBPlusTree::iterator_test_5() {
	typedef stx::btree_set<unsigned int,
			std::less<unsigned int>, struct traits_nodebug> btree_type;

	btree_type set;

	unsigned int maxnum = 100;

	for(unsigned int i = 0; i < maxnum; ++i)
	{
		set.insert(i);
	}

	{
		btree_type::iterator it;

		it = set.begin();
		it--;
		assert( it == set.begin() );

		it = set.begin();
		--it;
		assert( it == set.begin() );

		it = set.end();
		it++;
		assert( it == set.end() );

		it = set.end();
		++it;
		assert( it == set.end() );
	}

	{
		btree_type::const_iterator it;

		it = set.begin();
		it--;
		assert( it == set.begin() );

		it = set.begin();
		--it;
		assert( it == set.begin() );

		it = set.end();
		it++;
		assert( it == set.end() );

		it = set.end();
		++it;
		assert( it == set.end() );
	}

	{
		btree_type::reverse_iterator it;

		it = set.rbegin();
		it--;
		assert( it == set.rbegin() );

		it = set.rbegin();
		--it;
		assert( it == set.rbegin() );

		it = set.rend();
		it++;
		assert( it == set.rend() );

		it = set.rend();
		++it;
		assert( it == set.rend() );
	}

	{
		btree_type::const_reverse_iterator it;

		it = set.rbegin();
		it--;
		assert( it == set.rbegin() );

		it = set.rbegin();
		--it;
		assert( it == set.rbegin() );

		it = set.rend();
		it++;
		assert( it == set.rend() );

		it = set.rend();
		++it;
		assert( it == set.rend() );
	}
}

void UnitTestBPlusTree::iterator_test_6() {

	cout << "Unit Test \"void UnitTestBPlusTree::iterator_test_6()\"" << endl;

	typedef stx::btree_set<unsigned int,
			std::greater<unsigned int>, struct traits_nodebug> btree_type;

	btree_type set;

	std::set<size_t, std::greater<size_t> > std_set;

	unsigned int maxnum = 20000;	// it takes a lot of time to insert 3000+ objects into the b+tree why??

	cout << "Inserting elements into the stl red-black tree" << endl;
	for(unsigned int i = 0; i < maxnum; ++i) {
		std_set.insert(i);
	}

	cout << "Inserting elements into the b+ tree" << endl;
	for(unsigned int i = 0; i < maxnum; ++i) {
		set.insert(i);
	}

	cout << "Iterating over all elements of the b+ tree" << endl;
	btree_type::iterator it;
	for ( it = set.begin(); it != set.end(); ++it ) {
		//cout << *it << endl;
	}

	cout << "Finding an element in the b+ tree" << endl;
	size_t key = 11;	//9
	it = set.find(key);
	if ( it == set.end() )
		cout << "Key: " << key << " is not found in the set" << endl;
	else
		cout << "Key: " << key << " is found in the set" << endl;
}

