/*
 * CBinarySearchAlgo.cpp
 *
 *  Created on: Oct 1, 2011
 *      Author: data
 */

#include "CBinarySearchAlgo.h"

using namespace std;

CBinarySearchAlgo::CBinarySearchAlgo() {

}

CBinarySearchAlgo::~CBinarySearchAlgo() {

}

pair<streampos, bool> CBinarySearchAlgo::find(ifstream& binary_file_stream,
		ifstream& binary_index_file_stream, streampos index_record_length, std::string const& string_to_match) {

	pair<streampos, bool> match_result = make_pair(0, false);
	streampos file_size;

	if ( binary_file_stream.is_open() && binary_index_file_stream.is_open() ) {
		// getting the file size
		long begin = binary_index_file_stream.tellg();
		binary_index_file_stream.seekg( 0, ios::end );
		file_size = binary_index_file_stream.tellg() - begin;

		streampos end_index = file_size - index_record_length;
		cout << "file size = " << file_size << " : end index = " << end_index << endl;

		binary_index_file_stream.seekg(0, ios::beg);

		match_result = binary_search_recursively(binary_file_stream, binary_index_file_stream, index_record_length,
				string_to_match, 0, end_index);
	} else {
		cout << "Couldn't open either one of the files " << endl;
	}

	binary_file_stream.close();
	binary_index_file_stream.close();

	return match_result;
}

/**
 * It is decided upon that an index record for a word has the following components:
 * - file_offset (size_t)
 * - term_length (unsigned_short)
 *
 * @param in_stream is the sorted binary word dictionary
 * @param binary_index_file_stream is the binary index file corresponding to the dictionary
 * @param the length of a record (set of pointers) in the index file for the dictionary
 * @param string_to_match is the string to match within the dictionary
 * @param the beginning index in the dictionary index file - used for binary search
 * @param the end index in the dictionary index file - used for binary search
 * @param the size of the index file
 *
 * @note Note that the dictionary is a binary file of sorted words
 */
pair<streampos, bool> CBinarySearchAlgo::binary_search_recursively(
		ifstream& in_stream,
		ifstream& binary_index_file_stream,
		streampos index_record_length,
		string const& string_to_match,
		streampos begin_index,
		streampos end_index) {

	streampos index_of_match = 0;
	bool found = false;

	if ( end_index < begin_index ) {
		return make_pair(-1, found);
	}

	streampos index_difference = (end_index - begin_index)/2;
	streampos middle_index = begin_index + index_difference + index_difference%index_record_length;

	string begin_word = binary_search_string(in_stream, binary_index_file_stream, begin_index);
	string end_word = binary_search_string(in_stream, binary_index_file_stream, end_index);
	string middle_word = binary_search_string(in_stream, binary_index_file_stream, middle_index);

	cout << endl;
	cout << "bi = " << begin_index << " : mi = " << middle_index << " : ei = " << end_index << endl;
	cout << "bw = " << begin_word << " : mw = " << middle_word << " : ew = " << end_word << " | " << string_to_match << endl;

	// another base case
	if ( middle_index == end_index && middle_word != string_to_match )
		return make_pair(-1, false);

	// found a match
	if ( middle_word == string_to_match ) {
		return make_pair(middle_index, true);
		// else recurse
	} else if ( middle_word > string_to_match ) {
		end_index = middle_index;
		return binary_search_recursively(in_stream, binary_index_file_stream, index_record_length,
				string_to_match, begin_index, end_index);
	} else {
		begin_index = middle_index;
		return binary_search_recursively(in_stream, binary_index_file_stream, index_record_length,
				string_to_match, begin_index, end_index);
	}

	return make_pair(-1, found);
}

/**
 * returns the word in the word dictionary file corresponding to the offset in the dictionary's index file
 */
string CBinarySearchAlgo::binary_search_string(ifstream& in_stream, ifstream& binary_index_file_stream, streampos pos_in_index) {
	size_t file_offset;
	unsigned short term_length;
	binary_index_file_stream.seekg(pos_in_index, ios::beg);
	binary_index_file_stream.read(reinterpret_cast<char*>(&file_offset), sizeof(size_t));
	binary_index_file_stream.read(reinterpret_cast<char*>(&term_length), sizeof(unsigned short));

	//cout << "file_offset = " << file_offset << " : term_length = " << term_length << endl;
	char buffer[term_length+1];
	in_stream.seekg(file_offset, ios::beg);
	in_stream.read(buffer, term_length);
	buffer[term_length] = '\0';
	string word(buffer);
	return word;
}

