/*
 * CDictionary.cpp
 *
 *  Created on: Oct 1, 2011
 *      Author: data
 */

#include "CDictionary.h"

using namespace std;

CDictionary::CDictionary() {
	assert(_term_container.size() == 0);
	_current_container_size = 0;
	_current_num_of_bytes = 0;
	_file_record_delimiter[0] = 0x0A;
	_max_read_ahead_buffer_size = 64;
	_max_term_size = 32;
	_dictionary_index_file = "";
}

CDictionary::~CDictionary() {

}

void CDictionary::add_term(string term) {
	_term_container.push_back(term);
	++_current_container_size;
	_current_num_of_bytes += term.length();

	// TODO: if the container reaches a specified size, dump the container and start afresh
}

/**
 *
 */
void CDictionary::dump_container_to_binary_file(string binary_file_name) {
	// sort the container
	sort(_term_container.begin(), _term_container.end(), less<string>() );

	ofstream term_string_file_stream_handle;
	ofstream sorted_term_index_file_handle;
	_dictionary_index_file = binary_file_name+".idx.bin";

	term_string_file_stream_handle.open (binary_file_name.c_str(), ios::out | ios::trunc | ios::binary);
	sorted_term_index_file_handle.open (_dictionary_index_file.c_str(), ios::out | ios::trunc | ios::binary);

	if ( term_string_file_stream_handle.is_open() && sorted_term_index_file_handle.is_open() ) {
		size_t term_offset_in_file = 0;
		for ( size_t i = 0; i < _current_container_size; ++i ) {
			string term = _term_container[i];
			unsigned short term_length = term.length();

			term_string_file_stream_handle.write(const_cast<char *>(term.c_str()), term_length);	// do not write the null character
			term_string_file_stream_handle.write(reinterpret_cast<char *>(_file_record_delimiter), 1);

			sorted_term_index_file_handle.write(reinterpret_cast<char *>(&term_offset_in_file), sizeof(size_t));	// write the offset to the main file
			sorted_term_index_file_handle.write(reinterpret_cast<char *>(&term_length), sizeof(unsigned short));	// write the length of the term

			//cout << term << " : " << current_offset << " : " << (int)term_length << endl;	// diagnostics
			term_offset_in_file += term_length+1;
		}
	} else {
		cerr << "Counldn't open file " << binary_file_name << endl;
	}

	term_string_file_stream_handle.close();
	sorted_term_index_file_handle.close();
}

/**
 *
 */
void CDictionary::read_binary_file_into_container(string binary_file_name) {

	_term_container.clear();	// just a precaution

	ifstream dictionary_file_stream;

	// check dictionary index file exists or not
	ifstream dictionary_index_file_stream;
	dictionary_index_file_stream.open(_dictionary_index_file.c_str(), ios::in | ios::binary);

	if ( !dictionary_index_file_stream.is_open() ) {
		cerr << "The index file corresponding to the dictionary is not found .. cannot read dictionary" << endl;
		return;
	}

	size_t file_size;
	struct stat filestatus;
	stat( _dictionary_index_file.c_str(), &filestatus );
	file_size = filestatus.st_size;
	cout << "filesize = " << file_size << " bytes\n";

	dictionary_file_stream.open (binary_file_name.c_str(), ios::in | ios::binary);

	if ( dictionary_file_stream.is_open() ) {

		size_t term_offset_in_file;
		unsigned short term_length;
		char term_buffer[_max_term_size+1];

		dictionary_index_file_stream.seekg(0, ios::beg);

		while ( !dictionary_index_file_stream.eof() ) {

			// read from stream
			dictionary_index_file_stream.read(reinterpret_cast<char*>(&term_offset_in_file), sizeof(size_t));
			dictionary_index_file_stream.read(reinterpret_cast<char*>(&term_length), sizeof(unsigned short));

			//cout << term_offset_in_file << " : " << term_length << " : current container size = " << _current_container_size << endl;

			dictionary_file_stream.seekg((streampos)term_offset_in_file, ios::beg);
			dictionary_file_stream.read(term_buffer, term_length);
			term_buffer[term_length] = '\0';
			string term(term_buffer);
			//cout << "term = " << term << endl;
			_term_container.push_back(term);
		}

	} else {
		cout << "Counldn't open file " << binary_file_name << endl;
	}

	dictionary_index_file_stream.close();
	dictionary_file_stream.close();

}

/**
 *
 */
std::pair<std::streampos, bool> CDictionary::find_term(string binary_file_name, string term) {

	// check dictionary index file exists or not
	ifstream dictionary_index_file_stream;
	dictionary_index_file_stream.open(_dictionary_index_file.c_str(), ios::in | ios::binary);

	if ( !dictionary_index_file_stream.is_open() ) {
		cerr << "The index file corresponding to the dictionary is not found .. cannot read dictionary index" << endl;
		return make_pair(-1, false);
	}

	size_t index_record_length = sizeof(size_t)+sizeof(unsigned short);
	ifstream ifs;
	ifs.open(binary_file_name.c_str(), ios::in | ios::binary);
	if ( ifs.is_open() ) {
		return _binary_search_algorithm.find(ifs, dictionary_index_file_stream, index_record_length, term);
	} else {
		cout << "Cannot open file " << binary_file_name << endl;
		return make_pair(-1, false);
	}
}

/**
 *
 */
void CDictionary::print_container() {
	cout << "Printing container ... " << endl;
	_current_container_size = _term_container.size();
	for ( size_t i = 0; i < _current_container_size; ++i )
		cout << _term_container[i] << endl;
	cout << "----------------------" << endl;

}
