/*
 * CDictionary.h
 *
 *  Created on: Oct 1, 2011
 *      Author: data
 */

#ifndef CDICTIONARY_H_
#define CDICTIONARY_H_

#include <string>
#include <ios>
#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm>
#include <cassert>
#include <vector>
#include <sys/stat.h>
#include <sys/types.h>
#include "CBinarySearchAlgo.h"

class CDictionary {

private:
	// class members
	char _file_record_delimiter[1];
	size_t _max_num_of_entries;
	size_t _current_container_size;
	long _current_num_of_bytes;
	std::vector<std::string> _term_container;
	size_t _max_read_ahead_buffer_size;
	size_t _max_term_size;
	std::string _dictionary_index_file;

	// algorithms
	CBinarySearchAlgo _binary_search_algorithm;

public:
	CDictionary();
	virtual ~CDictionary();

	inline void set_file_record_delimiter(char record_delimiter) { _file_record_delimiter[0] = record_delimiter; }
	inline char get_file_record_delimiter() { return _file_record_delimiter[0]; }

	inline void set_max_read_ahead_buffer_size(size_t max_read_ahead_buffer_size) { _max_read_ahead_buffer_size = max_read_ahead_buffer_size; }
	inline size_t get_max_read_ahead_buffer_size() { return _max_read_ahead_buffer_size; }

	inline void set_max_term_size(size_t max_term_size) { _max_term_size = max_term_size; }
	inline size_t get_max_term_size() { return _max_term_size; }

	inline size_t get_current_container_size() { return _term_container.size(); }

	void add_term(std::string term);
	void dump_container_to_binary_file(std::string binary_file_name);
	void read_binary_file_into_container(std::string binary_file_name);
	std::pair<std::streampos, bool> find_term(std::string binary_file_name, std::string term);
	void print_container();
};

#endif /* CDICTIONARY_H_ */
