/*
 * CHeap.h
 *
 *  Created on: Aug 31, 2010
 *      Author: data
 */

#ifndef CHEAP_H_
#define CHEAP_H_

#include <iostream>
#include <algorithm>
#include <functional>
#include <vector>
#include <istream>
#include <ostream>
#include <cmath>
#include <cassert>

template<class T, class compare = std::less<T> >
class CHeap {

private:
	std::vector<T> _heap_vec;
	size_t _heap_size;
	size_t _vec_length;

public:
	/**
	 * @brief constructor
	 */
	CHeap() { _heap_size = 0; assert(_heap_vec.size() == _heap_size); }
	/**
	 * @brief destructor
	 */
	~CHeap() {}

	/**
	 * @brief returns the parent of "index" i in the heap array
	 * @param i
	 */
	size_t parent(size_t i);

	/**
	 * @brief returns the left child of a parent "index" i in the heap array
	 * @param i
	 */
	size_t left(size_t i);

	/**
	 * @brief returns the right child of a parent "index" i in the heap array
	 * @param i
	 */
	size_t right(size_t i);

	/**
	 * @brief max-heapify from an index i
	 * @param i
	 */
	void heapify(size_t i);
	/**
	 * @brief
	 * @param val
	 */
	void push_back(T& val);
	/**
	 * @brief
	 */
	void build_heap();
	/**
	 * @brief
	 */
	void sort();

	/**
	 *
	 */
	inline T& operator[] (size_t i) {	return _heap_vec.at(i);	}
	/**
	 *
	 */
	inline size_t size() { return _heap_size; }
	/**
	 *
	 */
	inline size_t length() { return _vec_length; }

};

/**
 * @brief
 * @param i
 * @note array indexing starts at 0
 */
template<class T, class compare >
size_t CHeap<T, compare >::parent(size_t i) {
	return (i-1)>>1;
}

/**
 * @brief
 * @param i
 * @note array indexing starts at 0
 */
template<class T, class compare >
size_t CHeap<T, compare>::left(size_t i) {
	return (i<<1)+1;
}

/**
 * @brief
 * @param i
 * @note array indexing starts at 0
 */
template<class T, class compare >
size_t CHeap<T, compare>::right(size_t i) {
	return (i<<1)+1 +1;
}

/**
 * @brief
 * @param i
 * @note array indexing starts at 0
 */
template<class T, class compare >
void CHeap<T, compare>::heapify(size_t i) {

	size_t l = left(i);
	size_t r = right(i);
	size_t selected;
	//std::cout << "i = " << i << " left = " << l << " right = " << r << std::endl;
	if ( (l < _heap_size) && (compare()(_heap_vec.at(i), _heap_vec.at(l))) ) {
		selected = l;
		//if ( compare()(_heap_vec.at(i), _heap_vec.at(l)) ) std::cout<<"il:true:" << std::endl; else  std::cout<<"il:false:" << std::endl;
	}
	else selected = i;
	if ( (r < _heap_size) && (compare()(_heap_vec.at(selected), _heap_vec.at(r))) ) {
		selected = r;
		//if ( compare()(_heap_vec.at(selected), _heap_vec.at(r)) ) std::cout<<"sr:true:" << std::endl; else  std::cout<<"sr:false:" << std::endl;
	}
	if ( selected != i ) {
		T temp = _heap_vec.at(i);
		_heap_vec.at(i) = _heap_vec.at(selected);
		_heap_vec.at(selected) = temp;
		heapify(selected);
	}
	//std::cout<<_heap_size<<':'<<selected<<':'<<i<<std::endl;
	return;
}

/**
 * @brief adds an element of type T to the heap vector
 * @note complexity is O(n)
 */
template<class T, class compare >
void CHeap<T, compare>::push_back(T& val) {
	_heap_vec.push_back(val);
	_heap_size = _heap_vec.size();
	_vec_length = _heap_size;
}

/**
 * @brief builds a heap
 * @note complexity is O(n)
 */
template<class T, class compare >
void CHeap<T, compare>::build_heap() {

	for ( int i = (_heap_size-1)/2; i >= 0; --i )
		heapify((size_t)i);
}

/**
 * @brief sort the elements in the heap
 * @note complexity is O(nlogn)
 */
template<class T, class compare >
void CHeap<T, compare>::sort() {
	build_heap();

	for ( size_t i = _heap_size-1; i >= 1; --i ) {
		T temp = _heap_vec[0];
		_heap_vec[0] = _heap_vec[i];
		_heap_vec[i] = temp;
		--_heap_size;
		heapify(0);
	}

}

#endif /* CHEAP_H_ */
